
//
//  ZDGradientView.m
//  gradient
//
//  Created by 张东 on 2017/4/27.
//  Copyright © 2017年 张东. All rights reserved.
//

#import "ZDGradientView.h"
#import "UIColor+Extension.h"

@interface ZDGradientView ()
// 外圈颜色
@property (nonatomic, strong) NSMutableArray *colors;
// 外圈
@property (nonatomic, weak) CAGradientLayer *gradientLayer;
//中圈的梯度
@property (nonatomic, weak) CAGradientLayer *centerGradientLayer;
//中圈的轨道
@property (nonatomic, weak) CAShapeLayer *shapeLaye;
// 内圈
@property (nonatomic, weak) CAShapeLayer *insideLayer;

@end

@implementation ZDGradientView

- (NSMutableArray *)colors {
    if (!_colors) {
        
        _colors = [NSMutableArray array];
        NSArray *arr = @[@0xffff0000,@0xffffff00, @0xff00ff00,@0xff00ffff,@0xff0000ff,@0xffff00ff];
        [arr enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            
            UIColor *hexColor = [UIColor colorWithHex:(long)obj];
            [_colors addObject: (__bridge id)hexColor.CGColor];
            
        }];
        
        
    }
    return _colors;
}

- (CAGradientLayer *)gradientLayer {
    if (!_gradientLayer) {
        
        
        // 创建一个shapeLayer 类
        CAShapeLayer *arc = [CAShapeLayer layer];
        // 设置shapeLayer的路径
        arc.path = [UIBezierPath bezierPathWithArcCenter:CGPointMake(self.frame.size.width * 0.5, self.frame.size.height * 0.5) radius:150.0 startAngle:(150.0 * M_PI) / 180 endAngle:(390.0  * M_PI) / 180 clockwise:YES].CGPath;
        arc.fillColor = [UIColor clearColor].CGColor;
        arc.strokeColor = [UIColor lightGrayColor].CGColor;
        arc.lineWidth = 15;
        arc.lineCap = kCALineCapButt;
        arc.lineDashPattern = [NSArray arrayWithObjects:@15, @1, nil];
        CAGradientLayer *gradientlayer = [CAGradientLayer layer];
        gradientlayer.frame = self.frame;
        gradientlayer.colors = self.colors;
        gradientlayer.mask = arc;
        gradientlayer.startPoint = CGPointMake(0, 0.5);
        gradientlayer.endPoint = CGPointMake(1, 0.5);
        _gradientLayer = gradientlayer;
    }
    return _gradientLayer;
}

- (CAShapeLayer *)shapeLaye {
    
    if (!_shapeLaye) {
        CAShapeLayer *shapeLayer = [CAShapeLayer layer];
        shapeLayer.path = [UIBezierPath bezierPathWithArcCenter:CGPointMake(self.frame.size.width * 0.5, self.frame.size.height * 0.5) radius:120.0 startAngle:(150 * M_PI) / 180 endAngle:(390 * M_PI) / 180 clockwise:1].CGPath;
        shapeLayer.fillColor = [UIColor clearColor].CGColor;
        shapeLayer.strokeColor = [UIColor lightGrayColor].CGColor;
        shapeLayer.lineWidth = 4;
        _shapeLaye = shapeLayer;
    }
    return _shapeLaye;
}

- (CAShapeLayer *)insideLayer {
    if (!_insideLayer) {
        
        CAShapeLayer *insideArc = [CAShapeLayer layer];
        insideArc.path = [UIBezierPath bezierPathWithArcCenter:CGPointMake(self.frame.size.width * 0.5, self.frame.size.height * 0.5) radius:90.0 startAngle:(150 * M_PI) / 180 endAngle:(390 * M_PI) / 180 clockwise:YES].CGPath;
        insideArc.lineWidth = 2;
        insideArc.lineDashPattern = @[@5,@1];
        insideArc.fillColor = [UIColor clearColor].CGColor;
        insideArc.strokeColor = [UIColor lightGrayColor].CGColor;
        _insideLayer = insideArc;
    }
    return _insideLayer;
}

- (void)setPercen:(NSInteger)percen {
    
    _percen = percen;
    
    [self.layer addSublayer:self.gradientLayer];
    [self.layer addSublayer:self.shapeLaye];
    [self.layer addSublayer:self.insideLayer];
    [self progrossPercen];
}

- (void)progrossPercen {
    
    CAShapeLayer *centerArc = [CAShapeLayer layer];
    centerArc.path = [UIBezierPath bezierPathWithArcCenter:CGPointMake(self.frame.size.width * 0.5, self.frame.size.height * 0.5) radius:120 startAngle:(150 * M_PI) / 180  endAngle:(((self.percen - 150) / 800.0 * 240 + 150) * M_PI) / 180 clockwise:1].CGPath;
    centerArc.fillColor = [UIColor clearColor].CGColor;
    centerArc.strokeColor = [UIColor lightGrayColor].CGColor;
    centerArc.lineWidth = 4;
    CABasicAnimation *drawAnimation = [CABasicAnimation animationWithKeyPath:@"strokeEnd"];
    drawAnimation.duration            = 5.0;
    drawAnimation.repeatCount         = 1.0;
    drawAnimation.removedOnCompletion = NO;
    drawAnimation.fromValue = [NSNumber numberWithInt:0];
    drawAnimation.toValue = [NSNumber numberWithInt:10];
    drawAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseIn];
    [centerArc addAnimation:drawAnimation forKey:@"drawCircleAnimat"];
    CAGradientLayer *changeLayer = [CAGradientLayer layer];
    changeLayer.frame = self.frame;
    changeLayer.colors = self.colors;
    changeLayer.startPoint = CGPointMake(0,0.5);
    changeLayer.endPoint = CGPointMake(1,0.5);
    changeLayer.mask = centerArc;
    [self.layer addSublayer:changeLayer];
}


@end
